@@ -1,12 +1,12 @@
 /* misc.c -  miscellaneous functions
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003,
- *               2004, 2005 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+ *               2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -288,7 +287,7 @@ checksum_mpi( MPI a )
     nbits = mpi_get_nbits(a);
     csum = checksum_u16( nbits );
     csum += checksum( buffer, nbytes );
-    m_free( buffer );
+    xfree( buffer );
     return csum;
 }
 
@@ -316,6 +315,10 @@ print_pubkey_algo_note( int algo )
 		   pubkey_algo_to_string(algo));
 	}
     }
+  else if (algo == 20)
+    {
+      log_info (_("WARNING: Elgamal sign+encrypt keys are deprecated\n"));
+    }
 }
 
 void
@@ -347,8 +350,7 @@ print_digest_algo_note( int algo )
 	}
     }
   else if(algo==DIGEST_ALGO_MD5)
-    log_info(_("WARNING: digest algorithm %s is deprecated\n"),
-	     digest_algo_to_string(algo));
+    md5_digest_warn (1);
 }
 
 /* Return a string which is used as a kind of process ID */
@@ -393,6 +395,9 @@ openpgp_cipher_test_algo( int algo )
 int
 openpgp_pk_test_algo( int algo, unsigned int usage_flags )
 {
+    /* Dont't allow type 20 keys unless in rfc2440 mode.  */
+    if (!RFC2440 && algo == 20)
+        return G10ERR_PUBKEY_ALGO;
     if( algo < 0 || algo > 110 )
 	return G10ERR_PUBKEY_ALGO;
     return check_pubkey_algo2( algo, usage_flags );
@@ -406,19 +411,26 @@ openpgp_pk_algo_usage ( int algo )
     /* they are hardwired in gpg 1.0 */
     switch ( algo ) {    
       case PUBKEY_ALGO_RSA:
-          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC | PUBKEY_USAGE_AUTH;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG | PUBKEY_USAGE_ENC | PUBKEY_USAGE_AUTH;
           break;
       case PUBKEY_ALGO_RSA_E:
           use = PUBKEY_USAGE_ENC;
           break;
       case PUBKEY_ALGO_RSA_S:
-          use = PUBKEY_USAGE_SIG;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG;
+          break;
+      case PUBKEY_ALGO_ELGAMAL:
+          /* Allow encryption with type 20 keys if RFC-2440 compliance
+             has been selected.  Signing is broken thus we won't allow
+             this.  */  
+          if (RFC2440)
+            use = PUBKEY_USAGE_ENC;
           break;
       case PUBKEY_ALGO_ELGAMAL_E:
           use = PUBKEY_USAGE_ENC;
           break;
       case PUBKEY_ALGO_DSA:  
-          use = PUBKEY_USAGE_SIG | PUBKEY_USAGE_AUTH;
+          use = PUBKEY_USAGE_CERT | PUBKEY_USAGE_SIG | PUBKEY_USAGE_AUTH;
           break;
       default:
           break;
@@ -444,14 +456,48 @@ idea_cipher_warn(int show)
   if(!warned || show)
     {
       log_info(_("the IDEA cipher plugin is not present\n"));
-      log_info(_("please see http://www.gnupg.org/why-not-idea.html "
-		 "for more information\n"));
+      log_info(_("please see %s for more information\n"),
+               "http://www.gnupg.org/faq/why-not-idea.html");
       warned=1;
     }
 }
 #endif
 
-static unsigned long get_signature_count(PKT_secret_key *sk)
+/* Print a warning if the md5 digest algorithm has been used.  This
+   warning is printed only once unless SHOW is used. */
+void
+md5_digest_warn (int show)
+{
+  static int warned = 0;
+
+  if (!warned || show)
+    {
+      log_info (_("WARNING: digest algorithm %s is deprecated\n"),
+                digest_algo_to_string (DIGEST_ALGO_MD5));
+      log_info (_("please see %s for more information\n"),
+                "http://www.gnupg.org/faq/weak-digest-algos.html");
+      warned = 1;
+    }
+}
+
+
+void
+not_in_gpg1_notice (void)
+{
+  static int warned = 0;
+
+  if (!warned)
+    {
+      log_info (_("NOTE: This feature is not available in %s\n"), "GnuPG 1.x");
+      log_info (_("please see %s for more information\n"),
+                "http://www.gnupg.org/faq/features-not-in-gnupg-1.html");
+      warned = 1;
+    }
+}
+
+
+static unsigned long 
+get_signature_count(PKT_secret_key *sk)
 {
 #ifdef ENABLE_CARD_SUPPORT
   if(sk && sk->is_protected && sk->protect.s2k.mode==1002)
@@ -467,7 +513,7 @@ static unsigned long get_signature_count(PKT_secret_key *sk)
   return 0;
 }
 
-/* Expand %-strings.  Returns a string which must be m_freed.  Returns
+/* Expand %-strings.  Returns a string which must be xfreed.  Returns
    NULL if the string cannot be expanded (too large). */
 char *
 pct_expando(const char *string,struct expando_args *args)
@@ -490,8 +536,6 @@ pct_expando(const char *string,struct expando_args *args)
 
   while(*ch!='\0')
     {
-      char *str=NULL;
-
       if(!done)
 	{
 	  /* 8192 is way bigger than we'll need here */
@@ -499,7 +543,7 @@ pct_expando(const char *string,struct expando_args *args)
 	    goto fail;
 
 	  maxlen+=1024;
-	  ret=m_realloc(ret,maxlen);
+	  ret=xrealloc(ret,maxlen);
 	}
 
       done=0;
@@ -570,7 +614,7 @@ pct_expando(const char *string,struct expando_args *args)
 		    else if(args->sk->main_keyid[0] || args->sk->main_keyid[1])
 		      {
 			PKT_public_key *pk=
-			  m_alloc_clear(sizeof(PKT_public_key));
+			  xmalloc_clear(sizeof(PKT_public_key));
 
 			if(get_pubkey_fast(pk,args->sk->main_keyid)==0)
 			  fingerprint_from_pk(pk,array,&len);
@@ -600,22 +644,46 @@ pct_expando(const char *string,struct expando_args *args)
 	      }
 	      break;
 
-	    case 't': /* e.g. "jpg" */
-	      str=image_type_to_string(args->imagetype,0);
-	      /* fall through */
-
-	    case 'T': /* e.g. "image/jpeg" */
-	      if(str==NULL)
-		str=image_type_to_string(args->imagetype,2);
-
-	      if(idx+strlen(str)<maxlen)
+	    case 'v': /* validity letters */
+	      if(args->validity_info && idx+1<maxlen)
 		{
-		  strcpy(&ret[idx],str);
-		  idx+=strlen(str);
+		  ret[idx++]=args->validity_info;
+		  ret[idx]='\0';
 		  done=1;
 		}
 	      break;
 
+	      /* The text string types */
+	    case 't':
+	    case 'T':
+	    case 'V':
+	      {
+		const char *str=NULL;
+
+		switch(*(ch+1))
+		  {
+		  case 't': /* e.g. "jpg" */
+		    str=image_type_to_string(args->imagetype,0);
+		    break;
+		  
+		  case 'T': /* e.g. "image/jpeg" */
+		    str=image_type_to_string(args->imagetype,2);
+		    break;
+
+		  case 'V': /* e.g. "full", "expired", etc. */
+		    str=args->validity_string;
+		    break;
+		  }
+
+		if(str && idx+strlen(str)<maxlen)
+		  {
+		    strcpy(&ret[idx],str);
+		    idx+=strlen(str);
+		    done=1;
+		  }
+	      }
+	      break;
+
 	    case '%':
 	      if(idx+1<maxlen)
 		{
@@ -661,7 +729,7 @@ pct_expando(const char *string,struct expando_args *args)
   return ret;
 
  fail:
-  m_free(ret);
+  xfree(ret);
   return NULL;
 }
 
@@ -686,6 +754,15 @@ deprecated_warning(const char *configname,unsigned int configlineno,
   log_info(_("please use \"%s%s\" instead\n"),repl1,repl2);
 }
 
+
+void
+deprecated_command (const char *name)
+{
+  log_info(_("WARNING: \"%s\" is a deprecated command - do not use it\n"),
+           name);
+}
+
+
 const char *
 compress_algo_to_string(int algo)
 {
@@ -789,47 +866,64 @@ default_compress_algo(void)
 const char *
 compliance_option_string(void)
 {
-  switch(opt.compliance)
-    {
-    case CO_RFC2440:
-      return "--openpgp";
-    case CO_PGP2:
-      return "--pgp2";
-    case CO_PGP6:
-      return "--pgp6";
-    case CO_PGP7:
-      return "--pgp7";
-    case CO_PGP8:
-      return "--pgp8";
-    default:
-      return "???";
-    }
-}
+  char *ver="???";
 
-static const char *
-compliance_string(void)
-{
   switch(opt.compliance)
     {
-    case CO_RFC2440:
-      return "OpenPGP";
-    case CO_PGP2:
-      return "PGP 2.x";
-    case CO_PGP6:
-      return "PGP 6.x";
-    case CO_PGP7:
-      return "PGP 7.x";
-    case CO_PGP8:
-      return "PGP 8.x";
-    default:
-      return "???";
+    case CO_GNUPG:   return "--gnupg";
+    case CO_RFC4880: return "--openpgp";
+    case CO_RFC2440: return "--rfc2440";
+    case CO_RFC1991: return "--rfc1991";
+    case CO_PGP2:    return "--pgp2";
+    case CO_PGP6:    return "--pgp6";
+    case CO_PGP7:    return "--pgp7";
+    case CO_PGP8:    return "--pgp8";
     }
+
+  return ver;
 }
 
 void
 compliance_failure(void)
 {
-  log_info(_("this message may not be usable by %s\n"),compliance_string());
+  char *ver="???";
+
+  switch(opt.compliance)
+    {
+    case CO_GNUPG:
+      ver="GnuPG";
+      break;
+
+    case CO_RFC4880:
+      ver="OpenPGP";
+      break;
+
+    case CO_RFC2440:
+      ver="OpenPGP (older)";
+      break;
+
+    case CO_RFC1991:
+      ver="old PGP";
+      break;
+
+    case CO_PGP2:
+      ver="PGP 2.x";
+      break;
+
+    case CO_PGP6:
+      ver="PGP 6.x";
+      break;
+
+    case CO_PGP7:
+      ver="PGP 7.x";
+      break;
+
+    case CO_PGP8:
+      ver="PGP 8.x";
+      break;
+    }
+
+  log_info(_("this message may not be usable by %s\n"),ver);
   opt.compliance=CO_GNUPG;
 }
 
@@ -956,6 +1050,24 @@ parse_options(char *str,unsigned int *options,
 {
   char *tok;
 
+  if (str && !strcmp (str, "help"))
+    {
+      int i,maxlen=0;
+
+      /* Figure out the longest option name so we can line these up
+	 neatly. */
+      for(i=0;opts[i].name;i++)
+	if(opts[i].help && maxlen<strlen(opts[i].name))
+	  maxlen=strlen(opts[i].name);
+
+      for(i=0;opts[i].name;i++)
+        if(opts[i].help)
+	  printf("%s%*s%s\n",opts[i].name,
+		 maxlen+2-(int)strlen(opts[i].name),"",_(opts[i].help));
+
+      g10_exit(0);
+    }
+
   while((tok=optsep(&str)))
     {
       int i,rev=0;
@@ -1020,6 +1132,87 @@ parse_options(char *str,unsigned int *options,
 }
 
 
+/* Return a new malloced string by unescaping the string S.  Escaping
+   is percent escaping and '+'/space mapping.  A binary nul will
+   silently be replaced by a 0xFF. */
+char *
+unescape_percent_string (const unsigned char *s)
+{
+  char *buffer, *d;
+
+  buffer = d = xmalloc (strlen (s)+1);
+  while (*s)
+    {
+      if (*s == '%' && s[1] && s[2])
+        { 
+          s++;
+          *d = xtoi_2 (s);
+          if (!*d)
+            *d = '\xff';
+          d++;
+          s += 2;
+        }
+      else if (*s == '+')
+        {
+          *d++ = ' ';
+          s++;
+        }
+      else
+        *d++ = *s++;
+    }
+  *d = 0; 
+  return buffer;
+}
+
+
+/* Check whether the string has characters not valid in an RFC-822
+   address.  To cope with OpenPGP we ignore non-ascii characters
+   so that for example umlauts are legal in an email address.  An
+   OpenPGP user ID must be utf-8 encoded but there is no strict
+   requirement for RFC-822.  Thus to avoid IDNA encoding we put the
+   address verbatim as utf-8 into the user ID under the assumption
+   that mail programs handle IDNA at a lower level and take OpenPGP
+   user IDs as utf-8.  Note that we can't do an utf-8 encoding
+   checking here because in keygen.c this function is called with the
+   native encoding and native to utf-8 encoding is only done  later.  */
+int
+has_invalid_email_chars (const char *s)
+{
+  int at_seen=0;
+  const char *valid_chars=
+    "01234567890_-.abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
+
+  for ( ; *s; s++ ) 
+    {
+      if ( *s & 0x80 )
+        continue; /* We only care about ASCII.  */
+      if ( *s == '@' )
+        at_seen=1;
+      else if ( !at_seen && !( !!strchr( valid_chars, *s ) || *s == '+' ) )
+        return 1;
+      else if ( at_seen && !strchr( valid_chars, *s ) )
+        return 1;
+    }
+  return 0;
+}
+
+
+/* Check whether NAME represents a valid mailbox according to
+   RFC822. Returns true if so. */
+int
+is_valid_mailbox (const char *name)
+{
+  return !( !name
+            || !*name
+            || has_invalid_email_chars (name)
+            || string_count_chr (name,'@') != 1
+            || *name == '@'
+            || name[strlen(name)-1] == '@'
+            || name[strlen(name)-1] == '.'
+            || strstr (name, "..") );
+}
+
+
 /* This is a helper function to load a Windows function from either of
    one DLLs. */
 #ifdef HAVE_W32_SYSTEM
@@ -1027,7 +1220,7 @@ static HRESULT
 w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
 {
   static int initialized;
-  static HRESULT (* WINAPI func)(HWND,int,HANDLE,DWORD,LPSTR);
+  static HRESULT (WINAPI * func)(HWND,int,HANDLE,DWORD,LPSTR);
 
   if (!initialized)
     {
@@ -1101,3 +1294,85 @@ default_homedir (void)
 
   return dir;
 }
+
+
+/* Return the name of the libexec directory.  The name is allocated in
+   a static area on the first use.  This function won't fail. */
+const char *
+get_libexecdir (void)
+{
+#ifdef HAVE_W32_SYSTEM
+  static int got_dir;
+  static char dir[MAX_PATH+5];
+
+  if (!got_dir)
+    {
+      char *p;
+
+      if ( !GetModuleFileName ( NULL, dir, MAX_PATH) )
+        {
+          log_debug ("GetModuleFileName failed: %s\n", w32_strerror (0));
+          *dir = 0;
+        }
+      got_dir = 1;
+      p = strrchr (dir, DIRSEP_C);
+      if (p)
+        *p = 0;
+      else
+        {
+          log_debug ("bad filename `%s' returned for this process\n", dir);
+          *dir = 0; 
+        }
+    }
+
+  if (*dir)
+    return dir;
+  /* Fallback to the hardwired value. */
+#endif /*HAVE_W32_SYSTEM*/
+
+  return GNUPG_LIBEXECDIR;
+}
+
+/* Similar to access(2), but uses PATH to find the file. */
+int
+path_access(const char *file,int mode)
+{
+  char *envpath;
+  int ret=-1;
+
+  envpath=getenv("PATH");
+
+  if(!envpath
+#ifdef HAVE_DRIVE_LETTERS
+     || (((file[0]>='A' && file[0]<='Z')
+	  || (file[0]>='a' && file[0]<='z'))
+	 && file[1]==':')
+#else
+     || file[0]=='/'
+#endif
+     )
+    return access(file,mode);
+  else
+    {
+      /* At least as large as, but most often larger than we need. */
+      char *buffer=xmalloc(strlen(envpath)+1+strlen(file)+1);
+      char *split,*item,*path=xstrdup(envpath);
+
+      split=path;
+
+      while((item=strsep(&split,PATHSEP_S)))
+	{
+	  strcpy(buffer,item);
+	  strcat(buffer,"/");
+	  strcat(buffer,file);
+	  ret=access(buffer,mode);
+	  if(ret==0)
+	    break;
+	}
+
+      xfree(path);
+      xfree(buffer);
+    }
+
+  return ret;
+}
