@@ -1,6 +1,6 @@
 /* parse-packet.c  - read packets
- * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *               2007 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
+ *               2008 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -346,12 +346,19 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
 	    rc = G10ERR_INVALID_PACKET;
 	    goto leave;
 	}
-        if (pkttype == PKT_COMPRESSED) {
-             iobuf_set_partial_block_mode(inp, c & 0xff);
-             pktlen = 0;/* to indicate partial length */
-	     partial=1;
-        }
-        else {
+/* The follwing code has been here for ages (2002-08-30) but it is
+   clearly wrong: For example passing a 0 as second argument to
+   iobuf_set_partial_block_mode stops the partial block mode which we
+   definitely do not want.  Also all values < 224 or 255 are not
+   valid.  Let's disable it and put PKT_COMPRESSED into the list of
+   allowed packets with partial header until someone complains. */
+/*         if (pkttype == PKT_COMPRESSED) { */
+/*              iobuf_set_partial_block_mode(inp, c & 0xff); */
+/*              pktlen = 0;/\* to indicate partial length *\/ */
+/* 	     partial=1; */
+/*         } */
+/*         else { */
+        {
              hdr[hdrlen++] = c;
              if( c < 192 )
 	       pktlen = c;
@@ -384,20 +391,22 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
 	       }
              else
 	       {
-		 /* Partial body length.  Note that we handled
-		    PKT_COMPRESSED earlier. */
-		 if(pkttype==PKT_PLAINTEXT || pkttype==PKT_ENCRYPTED
-		    || pkttype==PKT_ENCRYPTED_MDC)
-		   {
-		     iobuf_set_partial_block_mode(inp, c & 0xff);
-		     pktlen = 0;/* to indicate partial length */
-		     partial=1;
-		   }
-		 else
-		   {
-		     log_error("%s: partial length for invalid"
-			       " packet type %d\n",iobuf_where(inp),pkttype);
-		     rc=G10ERR_INVALID_PACKET;
+		 /* Partial body length.  */
+		 switch (pkttype)
+                   {
+                   case PKT_PLAINTEXT:
+                   case PKT_ENCRYPTED:
+                   case PKT_ENCRYPTED_MDC:
+                   case PKT_COMPRESSED:
+		     iobuf_set_partial_block_mode (inp, (c & 0xff));
+		     pktlen = 0; /* Indicate partial length.  */
+		     partial= 1;
+		     break;
+
+                   default:
+		     log_error ("%s: partial length for invalid"
+                                " packet type %d\n", iobuf_where(inp),pkttype);
+		     rc = G10ERR_INVALID_PACKET;
 		     goto leave;
 		   }
 	       }
@@ -432,8 +441,8 @@ parse( IOBUF inp, PACKET *pkt, int onlykeypkts, off_t *retpos,
 	  }
       }
 
-    if (pktlen == 0xffffffff) {
-        /* with a some probability this is caused by a problem in the
+    if (pktlen == (unsigned long)(-1)) {
+        /* With some probability this is caused by a problem in the
          * the uncompressing layer - in some error cases it just loops
          * and spits out 0xff bytes. */
         log_error ("%s: garbled packet detected\n", iobuf_where(inp) );
@@ -1854,8 +1863,9 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	     * enlarge temp.
 	     */
 	    switch( sk->protect.algo ) {
-	      case 7: case 8: case 9: /* reserved for AES */
+	      case 7: case 8: case 9: /* AES */
 	      case 10: /* Twofish */
+	      case 11: case 12: case 13: /* Camellia */
 		sk->protect.ivlen = 16;
 		break;
 	      default:
