@@ -5,7 +5,7 @@
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -14,9 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef G10_IOBUF_H
@@ -36,6 +34,7 @@
 #define IOBUFCTRL_USER	    16
 
 typedef struct iobuf_struct *IOBUF;
+typedef struct iobuf_struct *iobuf_t;
 
 /* fixme: we should hide most of this stuff */
 struct iobuf_struct {
@@ -135,10 +134,10 @@ void iobuf_set_partial_block_mode( IOBUF a, size_t len );
 
 int iobuf_translate_file_handle ( int fd, int for_write );
 
-/* get a byte form the iobuf; must check for eof prior to this function
- * this function returns values in the range 0 .. 255 or -1 to indicate EOF
+/* Get a byte form the iobuf; must check for eof prior to this function.
+ * This function returns values in the range 0 .. 255 or -1 to indicate EOF
  * iobuf_get_noeof() does not return -1 to indicate EOF, but masks the
- * returned value to be in the range 0 ..255.
+ * returned value to be in the range 0..255.
  */
 #define iobuf_get(a)  \
      (	((a)->nofast || (a)->d.start >= (a)->d.len )?  \
