@@ -6,7 +6,7 @@
  *
  * GNUPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GNUPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef G10_UTIL_H
 #define G10_UTIL_H
@@ -143,6 +141,7 @@ char * make_basename(const char *filepath, const char *inputpath);
 char * make_dirname(const char *filepath);
 char *make_filename( const char *first_part, ... );
 int compare_filenames( const char *a, const char *b );
+int same_file_p (const char *name1, const char *name2);
 const char *print_fname_stdin( const char *s );
 const char *print_fname_stdout( const char *s );
 int is_file_compressed(const char *s, int *r_status);
@@ -150,6 +149,7 @@ int is_file_compressed(const char *s, int *r_status);
 /*-- miscutil.c --*/
 u32 make_timestamp(void);
 u32 scan_isodatestr( const char *string );
+u32 isotime2seconds (const char *string);
 const char *strtimevalue( u32 stamp );
 const char *strtimestamp( u32 stamp ); /* GMT */
 const char *isotimestamp( u32 stamp ); /* GMT with hh:mm:ss */
@@ -190,14 +190,10 @@ int set_native_charset( const char *newset );
 const char* get_native_charset(void);
 char *native_to_utf8( const char *string );
 char *utf8_to_native( const char *string, size_t length, int delim);
-int  check_utf8_string( const char *string );
+char *string_to_utf8 (const char *string);
 
 int ascii_isupper (int c);
 int ascii_islower (int c);
-int ascii_toupper (int c);
-int ascii_tolower (int c);
-int ascii_strcasecmp( const char *a, const char *b );
-int ascii_strncasecmp( const char *a, const char *b, size_t n);
 int ascii_memcasecmp( const char *a, const char *b, size_t n);
 
 #ifndef HAVE_STPCPY
@@ -244,11 +240,13 @@ char *read_w32_registry_string( const char *root,
 int write_w32_registry_string(const char *root, const char *dir,
                               const char *name, const char *value);
 
-/*-- strgutil.c --*/
-int vasprintf (char **result, const char *format, va_list args);
-int asprintf (char **buf, const char *fmt, ...);
 #endif /*_WIN32*/
 
+/*-- strgutil.c --*/
+char *xasprintf (const char *fmt, ...);
+char *xtryasprintf (const char *fmt, ...);
+
+
 /*-- pka.c --*/
 char *get_pka_info (const char *address, unsigned char *fpr);
 
@@ -256,6 +254,16 @@ char *get_pka_info (const char *address, unsigned char *fpr);
 int get_cert(const char *name,size_t max_size,IOBUF *iobuf,
 	     unsigned char **fpr,size_t *fpr_len,char **url);
 
+/*-- convert.c --*/
+int hex2bin (const char *string, void *buffer, size_t length);
+int hexcolon2bin (const char *string, void *buffer, size_t length);
+char *bin2hex (const void *buffer, size_t length, char *stringbuf);
+char *bin2hexcolon (const void *buffer, size_t length, char *stringbuf);
+const char *hex2str (const char *hexstring,
+                     char *buffer, size_t bufsize, size_t *buflen);
+char *hex2str_alloc (const char *hexstring, size_t *r_count);
+
+
 /**** other missing stuff ****/
 #ifndef HAVE_ATEXIT  /* For SunOS */
 #define atexit(a)    (on_exit((a),0))
