@@ -1,11 +1,11 @@
 /* mpicoder.c  -  Coder for the external representation of MPIs
- * Copyright (C) 1998, 1999 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -31,7 +30,7 @@
 #include "util.h"
 
 #ifdef M_DEBUG
-  #undef mpi_read
+#undef mpi_read
 #endif
 
 #define MAX_EXTERN_MPI_BITS 16384
@@ -55,7 +54,7 @@ mpi_write( IOBUF out, MPI a )
 
     p = buf = mpi_get_buffer( a, &n, NULL );
     rc = iobuf_write( out, p, n );
-    m_free(buf);
+    xfree(buf);
     return rc;
 }
 
@@ -74,31 +73,39 @@ mpi_read(IOBUF inp, unsigned *ret_nread, int secure)
 #endif
 {
     int c, i, j;
+    unsigned int nmax = *ret_nread;
     unsigned nbits, nbytes, nlimbs, nread=0;
     mpi_limb_t a;
-    MPI val = MPI_NULL;
+    MPI val = NULL;
 
+    if (nread == nmax)
+        goto overflow;
     if( (c = iobuf_get(inp)) == -1 )
 	goto leave;
+    nread++;
     nbits = c << 8;
+
+    if (nread == nmax)
+        goto overflow;
     if( (c = iobuf_get(inp)) == -1 )
 	goto leave;
+    nread++;
     nbits |= c;
+
     if( nbits > MAX_EXTERN_MPI_BITS ) {
-	log_error("mpi too large (%u bits)\n", nbits);
+	log_error("mpi too large for this implementation (%u bits)\n", nbits);
 	goto leave;
     }
-    nread = 2;
 
     nbytes = (nbits+7) / 8;
     nlimbs = (nbytes+BYTES_PER_MPI_LIMB-1) / BYTES_PER_MPI_LIMB;
-  #ifdef M_DEBUG
+#ifdef M_DEBUG
     val = secure? mpi_debug_alloc_secure( nlimbs, info )
 		: mpi_debug_alloc( nlimbs, info );
-  #else
+#else
     val = secure? mpi_alloc_secure( nlimbs )
 		: mpi_alloc( nlimbs );
-  #endif
+#endif
     i = BYTES_PER_MPI_LIMB - nbytes % BYTES_PER_MPI_LIMB;
     i %= BYTES_PER_MPI_LIMB;
     val->nbits = nbits;
@@ -107,6 +114,15 @@ mpi_read(IOBUF inp, unsigned *ret_nread, int secure)
     for( ; j > 0; j-- ) {
 	a = 0;
 	for(; i < BYTES_PER_MPI_LIMB; i++ ) {
+            if (nread == nmax) {
+#ifdef M_DEBUG
+                mpi_debug_free (val);
+#else
+                mpi_free (val);
+#endif
+                val = NULL;
+                goto overflow;
+            }
 	    a <<= 8;
 	    a |= iobuf_get(inp) & 0xff; nread++;
 	}
@@ -115,27 +131,28 @@ mpi_read(IOBUF inp, unsigned *ret_nread, int secure)
     }
 
   leave:
-    if( nread > *ret_nread )
-	log_bug("mpi crosses packet border\n");
-    else
-	*ret_nread = nread;
+    *ret_nread = nread;
+    return val;
+  overflow:
+    log_error ("mpi larger than indicated length (%u bytes)\n", nmax);
+    *ret_nread = nread;
     return val;
 }
 
 
 MPI
-mpi_read_from_buffer(byte *buffer, unsigned *ret_nread, int secure)
+mpi_read_from_buffer(byte *buffer, unsigned int *ret_nread, int secure)
 {
     int i, j;
     unsigned nbits, nbytes, nlimbs, nread=0;
     mpi_limb_t a;
-    MPI val = MPI_NULL;
+    MPI val = NULL;
 
     if( *ret_nread < 2 )
 	goto leave;
     nbits = buffer[0] << 8 | buffer[1];
     if( nbits > MAX_EXTERN_MPI_BITS ) {
-	log_error("mpi too large (%u bits)\n", nbits);
+	log_info ("mpi too large (%u bits)\n", nbits);
 	goto leave;
     }
     buffer += 2;
@@ -153,10 +170,19 @@ mpi_read_from_buffer(byte *buffer, unsigned *ret_nread, int secure)
     for( ; j > 0; j-- ) {
 	a = 0;
 	for(; i < BYTES_PER_MPI_LIMB; i++ ) {
-	    if( ++nread > *ret_nread )
-		log_bug("mpi larger than buffer\n");
-	    a <<= 8;
-	    a |= *buffer++;
+          if( ++nread > *ret_nread ) {
+              /* This (as well as the above error condition) may
+                 happen if we use this function to parse a decrypted
+                 MPI which didn't turn out to be a real MPI - possible
+                 because the supplied key was wrong but the OpenPGP
+                 checksum didn't caught it. */
+		log_info ("mpi larger than buffer\n");
+                mpi_free (val);
+                val = NULL;
+                goto leave;
+          }
+          a <<= 8;
+          a |= *buffer++;
 	}
 	i = 0;
 	val->d[j-1] = a;
@@ -252,7 +278,7 @@ mpi_print( FILE *fp, MPI a, int mode )
 {
     int i, n=0;
 
-    if( a == MPI_NULL )
+    if( a == NULL )
 	return fprintf(fp, "[MPI_NULL]");
     if( !mode ) {
 	unsigned int n1;
@@ -314,13 +340,13 @@ mpi_get_keyid( MPI a, u32 *keyid )
     }
     return a->nlimbs? (u32)(a->d[0] & 0xffffffff) : 0;
 #else
-  #error Make this function work with other LIMB sizes
+#error Make this function work with other LIMB sizes
 #endif
 }
 
 
 /****************
- * Return an m_alloced buffer with the MPI (msb first).
+ * Return an xmalloced buffer with the MPI (msb first).
  * NBYTES receives the length of this buffer. Caller must free the
  * return string (This function does return a 0 byte buffer with NBYTES
  * set to zero if the value of A is zero. If sign is not NULL, it will
@@ -339,8 +365,8 @@ do_get_buffer( MPI a, unsigned *nbytes, int *sign, int force_secure )
     *nbytes = n = a->nlimbs * BYTES_PER_MPI_LIMB;
     if (!n)
       n++; /* avoid zero length allocation */
-    p = buffer = force_secure || mpi_is_secure(a) ? m_alloc_secure(n)
-						  : m_alloc(n);
+    p = buffer = force_secure || mpi_is_secure(a) ? xmalloc_secure(n)
+						  : xmalloc(n);
 
     for(i=a->nlimbs-1; i >= 0; i-- ) {
 	alimb = a->d[i];
@@ -402,12 +428,12 @@ mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign )
     a->sign = sign;
 
     for(i=0, p = buffer+nbytes-1; p >= buffer+BYTES_PER_MPI_LIMB; ) {
-      #if BYTES_PER_MPI_LIMB == 4
+#if BYTES_PER_MPI_LIMB == 4
 	alimb  = (mpi_limb_t)*p-- ;
 	alimb |= (mpi_limb_t)*p-- <<  8 ;
 	alimb |= (mpi_limb_t)*p-- << 16 ;
 	alimb |= (mpi_limb_t)*p-- << 24 ;
-      #elif BYTES_PER_MPI_LIMB == 8
+#elif BYTES_PER_MPI_LIMB == 8
 	alimb  = (mpi_limb_t)*p--	;
 	alimb |= (mpi_limb_t)*p-- <<  8 ;
 	alimb |= (mpi_limb_t)*p-- << 16 ;
@@ -416,18 +442,18 @@ mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign )
 	alimb |= (mpi_limb_t)*p-- << 40 ;
 	alimb |= (mpi_limb_t)*p-- << 48 ;
 	alimb |= (mpi_limb_t)*p-- << 56 ;
-      #else
-	#error please implement for this limb size.
-      #endif
+#else
+#error please implement for this limb size.
+#endif
 	a->d[i++] = alimb;
     }
     if( p >= buffer ) {
-      #if BYTES_PER_MPI_LIMB == 4
+#if BYTES_PER_MPI_LIMB == 4
 	alimb  = *p--	    ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- <<  8 ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 16 ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 24 ;
-      #elif BYTES_PER_MPI_LIMB == 8
+#elif BYTES_PER_MPI_LIMB == 8
 	alimb  = (mpi_limb_t)*p-- ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- <<	8 ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 16 ;
@@ -436,12 +462,11 @@ mpi_set_buffer( MPI a, const byte *buffer, unsigned nbytes, int sign )
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 40 ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 48 ;
 	if( p >= buffer ) alimb |= (mpi_limb_t)*p-- << 56 ;
-      #else
-	#error please implement for this limb size.
-      #endif
+#else
+#error please implement for this limb size.
+#endif
 	a->d[i++] = alimb;
     }
     a->nlimbs = i;
     assert( i == nlimbs );
 }
-
