@@ -1,12 +1,12 @@
 /* iobuf.c  -  file handling
- * Copyright (C) 1998, 1999, 2000, 2001, 2003,
- *               2004 Free Software Foundation, Inc.
+ * Copyright (C) 1998, 1999, 2000, 2001, 2003, 2004, 2005, 2006, 2008,
+ *               2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -164,10 +162,11 @@ fd_cache_strcmp (const char *a, const char *b)
  * Invalidate (i.e. close) a cached iobuf or all iobufs if NULL is
  * used for FNAME.
  */
-static void
+static int
 fd_cache_invalidate (const char *fname)
 {
     CLOSE_CACHE cc;
+    int err=0;
 
     if (!fname) {
         if( DBG_IOBUF )
@@ -183,7 +182,7 @@ fd_cache_invalidate (const char *fname)
                 cc->fp = INVALID_FP;
             }
         }
-        return;
+        return err;
     }
 
     if( DBG_IOBUF )
@@ -194,16 +193,43 @@ fd_cache_invalidate (const char *fname)
             if( DBG_IOBUF )
                 log_debug ("                did (%s)\n", cc->fname);
 #ifdef HAVE_DOSISH_SYSTEM
-            CloseHandle (cc->fp);
+            if(CloseHandle (cc->fp)==0)
+	      err=-1;
 #else
-	    close(cc->fp);
+	    err=close(cc->fp);
 #endif
             cc->fp = INVALID_FP;
         }
     }
+
+    return err;
 }
 
+static int
+fd_cache_synchronize(const char *fname)
+{
+  int err=0;
 
+#ifndef HAVE_DOSISH_SYSTEM
+  CLOSE_CACHE cc;
+
+  if( DBG_IOBUF )
+    log_debug ("fd_cache_synchronize (%s)\n", fname);
+
+  for (cc=close_cache; cc; cc = cc->next )
+    {
+      if ( cc->fp != INVALID_FP && !fd_cache_strcmp (cc->fname, fname) )
+	{
+	  if( DBG_IOBUF )
+	    log_debug ("                did (%s)\n", cc->fname);
+
+	  err=fsync(cc->fp);
+	}
+    }
+#endif
+
+  return err;
+}
 
 static FILEP_OR_FD
 direct_open (const char *fname, const char *mode)
@@ -293,7 +319,7 @@ fd_cache_close (const char *fname, FILEP_OR_FD fp)
         close(fp);
 #endif
         if( DBG_IOBUF )
-            log_debug ("fd_cache_close (%p) real\n", (void*)fp);
+          log_debug ("fd_cache_close (%d) real\n", (int)fp);
         return;
     }
     /* try to reuse a slot */
@@ -885,7 +911,7 @@ print_chain( IOBUF a )
 						(byte*)&desc, &dummy_len );
 
 	log_debug("iobuf chain: %d.%d `%s' filter_eof=%d start=%d len=%d\n",
-		   a->no, a->subno, desc, a->filter_eof,
+		   a->no, a->subno, desc?desc:"?", a->filter_eof,
 		   (int)a->d.start, (int)a->d.len );
     }
 }
@@ -940,7 +966,8 @@ iobuf_close ( IOBUF a )
 	    log_error("iobuf_flush failed on close: %s\n", g10_errstr(rc));
 
 	if( DBG_IOBUF )
-	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno, a->desc );
+	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno,
+                      a->desc?a->desc:"?");
 	if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE,
 					 a->chain, NULL, &dummy_len)) )
 	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
@@ -1201,7 +1228,8 @@ iobuf_create( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: create `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: create `%s'\n", a->no, a->subno, 
+                  a->desc?a->desc:"?" );
 
     return a;
 }
@@ -1234,7 +1262,8 @@ iobuf_append( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: append `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: append `%s'\n", a->no, a->subno, 
+                  a->desc?a->desc:"?" );
 
     return a;
 }
@@ -1262,7 +1291,8 @@ iobuf_openrw( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: openrw `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: openrw `%s'\n", a->no, a->subno, 
+                  a->desc?a->desc:"?");
 
     return a;
 }
@@ -1274,7 +1304,8 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
     if ( cmd == 1 ) {  /* keep system filepointer/descriptor open */
         if( DBG_IOBUF )
             log_debug("iobuf-%d.%d: ioctl `%s' keep=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
+                      a? a->no:-1, a?a->subno:-1, 
+                      a&&a->desc?a->desc:"?", intval );
         for( ; a; a = a->chain )
             if( !a->chain && a->filter == file_filter ) {
                 file_filter_ctx_t *b = a->filter_ov;
@@ -1295,15 +1326,16 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
                       ptrval? (char*)ptrval:"[all]");
         if ( !a && !intval ) {
 #ifndef FILE_FILTER_USES_STDIO
-            fd_cache_invalidate (ptrval);
+	    return fd_cache_invalidate (ptrval);
 #endif
             return 0;
-        }
+	 }
     }
     else if ( cmd == 3 ) {  /* disallow/allow caching */
         if( DBG_IOBUF )
             log_debug("iobuf-%d.%d: ioctl `%s' no_cache=%d\n",
-                      a? a->no:-1, a?a->subno:-1, a?a->desc:"?", intval );
+                      a? a->no:-1, a?a->subno:-1, 
+                      a&&a->desc?a->desc:"?", intval );
         for( ; a; a = a->chain )
             if( !a->chain && a->filter == file_filter ) {
                 file_filter_ctx_t *b = a->filter_ov;
@@ -1318,6 +1350,23 @@ iobuf_ioctl ( IOBUF a, int cmd, int intval, void *ptrval )
             }
 #endif
     }
+    else if(cmd==4)
+      {
+	/* Do a fsync on the open fd and return any errors to the
+	   caller of iobuf_ioctl */
+        if( DBG_IOBUF )
+	  log_debug("iobuf-*.*: ioctl `%s' fsync\n",
+                      ptrval? (char*)ptrval:"<null>");
+
+	if(!a && !intval && ptrval)
+	  {
+#ifndef FILE_FILTER_USES_STDIO
+	    return fd_cache_synchronize (ptrval);
+#else
+	    return 0;
+#endif
+	  }
+      }
 
     return -1;
 }
@@ -1395,7 +1444,8 @@ iobuf_push_filter2( IOBUF a,
     f( ov, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &dummy_len );
 
     if( DBG_IOBUF ) {
-	log_debug("iobuf-%d.%d: push `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: push `%s'\n", a->no, a->subno, 
+                  a->desc?a->desc:"?" );
 	print_chain( a );
     }
 
@@ -1421,7 +1471,8 @@ pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	BUG();
 
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: pop `%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: pop `%s'\n", a->no, a->subno,
+                  a->desc?a->desc:"?" );
     if( !a->filter ) { /* this is simple */
 	b = a->chain;
 	assert(b);
@@ -1499,7 +1550,7 @@ underflow(IOBUF a)
 	    IOBUF b = a->chain;
 	    if( DBG_IOBUF )
 		log_debug("iobuf-%d.%d: pop `%s' in underflow\n",
-					a->no, a->subno, a->desc );
+                          a->no, a->subno, a->desc?a->desc:"?" );
 	    xfree(a->d.buf);
 	    xfree(a->real_fname);
 	    memcpy(a, b, sizeof *a);
@@ -1564,8 +1615,8 @@ underflow(IOBUF a)
 	    if( !len && a->chain ) {
 		IOBUF b = a->chain;
 		if( DBG_IOBUF )
-		    log_debug("iobuf-%d.%d: pop `%s' in underflow (!len)\n",
-					       a->no, a->subno, a->desc );
+		    log_debug("iobuf-%d.%d: pop in underflow (!len)\n",
+                              a->no, a->subno);
 		xfree(a->d.buf);
 		xfree(a->real_fname);
 		memcpy(a,b, sizeof *a);
