@@ -1,12 +1,12 @@
 /* strgutil.c -  string utilities
  * Copyright (C) 1994, 1998, 1999, 2000, 2001,
- *               2003, 2004, 2005 Free Software Foundation, Inc.
+ *               2003, 2004, 2005, 2009 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -48,6 +47,14 @@
 #include "memory.h"
 #include "i18n.h"
 #include "dynload.h"
+#include "estream-printf.h"
+
+/* Our xasprintf replacements are expected to work with our memory
+   allocator.  Let's test for this here.  */
+#if !defined(_ESTREAM_PRINTF_MALLOC) || !defined(_ESTREAM_PRINTF_FREE)
+#error Please define _ESTREAM_PRINTF_MALLOC and _FREE
+#endif
+
 
 
 #ifndef USE_GNUPG_ICONV
@@ -167,7 +174,7 @@ free_strlist( STRLIST sl )
 
     for(; sl; sl = sl2 ) {
 	sl2 = sl->next;
-	m_free(sl);
+	xfree(sl);
     }
 }
 
@@ -177,7 +184,7 @@ add_to_strlist( STRLIST *list, const char *string )
 {
     STRLIST sl;
 
-    sl = m_alloc( sizeof *sl + strlen(string));
+    sl = xmalloc( sizeof *sl + strlen(string));
     sl->flags = 0;
     strcpy(sl->d, string);
     sl->next = *list;
@@ -199,7 +206,7 @@ add_to_strlist2( STRLIST *list, const char *string, int is_utf8 )
     else {
 	char *p = native_to_utf8( string );
 	sl = add_to_strlist( list, p );
-	m_free( p );
+	xfree( p );
     }
     return sl;
 }
@@ -209,7 +216,7 @@ append_to_strlist( STRLIST *list, const char *string )
 {
     STRLIST r, sl;
 
-    sl = m_alloc( sizeof *sl + strlen(string));
+    sl = xmalloc( sizeof *sl + strlen(string));
     sl->flags = 0;
     strcpy(sl->d, string);
     sl->next = NULL;
@@ -233,7 +240,7 @@ append_to_strlist2( STRLIST *list, const char *string, int is_utf8 )
     else {
 	char *p = native_to_utf8( string );
 	sl = append_to_strlist( list, p );
-	m_free( p );
+	xfree( p );
     }
     return sl;
 }
@@ -266,11 +273,11 @@ pop_strlist( STRLIST *list )
 
   if(sl)
     {
-      str=m_alloc(strlen(sl->d)+1);
+      str=xmalloc(strlen(sl->d)+1);
       strcpy(str,sl->d);
 
       *list=sl->next;
-      m_free(sl);
+      xfree(sl);
     }
 
   return str;
@@ -322,7 +329,7 @@ ascii_memistr( const char *buf, size_t buflen, const char *sub )
 
 /* Like strncpy() but copy at max N-1 bytes and append a '\0'.  With
  * N given as 0 nothing is copied at all. With DEST given as NULL
- * sufficient memory is allocated using m_alloc (note that m_alloc is
+ * sufficient memory is allocated using xmalloc (note that xmalloc is
  * guaranteed to succeed or to abort the process).  */
 char *
 mem2str( char *dest , const void *src , size_t n )
@@ -332,7 +339,7 @@ mem2str( char *dest , const void *src , size_t n )
 
     if( n ) {
 	if( !dest )
-	    dest = m_alloc( n ) ;
+	    dest = xmalloc( n ) ;
 	d = dest;
 	s = src ;
 	for(n--; n && *s; n-- )
@@ -536,7 +543,8 @@ set_native_charset( const char *newset )
                         "CP28597" "\0" "ISO-8859-7" "\0"
                         "CP28598" "\0" "ISO-8859-8" "\0"
                         "CP28599" "\0" "ISO-8859-9" "\0"
-                        "CP28605" "\0" "ISO-8859-15" "\0");
+                        "CP28605" "\0" "ISO-8859-15" "\0"
+			"CP65001" "\0" "UTF-8" "\0");
              *aliases;
              aliases += strlen (aliases) + 1, aliases += strlen (aliases) + 1)
           {
@@ -675,7 +683,7 @@ native_to_utf8( const char *string )
   
   if (no_translation)
     { /* Already utf-8 encoded. */
-      buffer = m_strdup (string);
+      buffer = xstrdup (string);
     }
   else if( !active_charset && !use_iconv) /* Shortcut implementation
                                              for Latin-1.  */
@@ -686,7 +694,7 @@ native_to_utf8( const char *string )
           if( *s & 0x80 )
             length++;
 	}
-      buffer = m_alloc( length + 1 );
+      buffer = xmalloc( length + 1 );
       for(p=buffer, s=string; *s; s++ )
         {
           if( *s & 0x80 )
@@ -720,7 +728,7 @@ native_to_utf8( const char *string )
           if ((*s & 0x80))
             length += 5; /* We may need up to 6 bytes for the utf8 output. */
         }
-      buffer = m_alloc (length + 1);
+      buffer = xmalloc (length + 1);
 
       inptr = string;
       inbytes = strlen (string);
@@ -754,7 +762,7 @@ native_to_utf8( const char *string )
           if( *s & 0x80 )
             length += 2; /* We may need up to 3 bytes. */
         }
-      buffer = m_alloc( length + 1 );
+      buffer = xmalloc( length + 1 );
       for(p=buffer, s=string; *s; s++ ) {
         if( *s & 0x80 ) {
           ushort val = active_charset[ *s & 0x7f ];
@@ -976,7 +984,7 @@ utf8_to_native( const char *string, size_t length, int delim )
 	    }
 	}
 	if( !buffer ) { /* allocate the buffer after the first pass */
-	    buffer = p = m_alloc( n + 1 );
+	    buffer = p = xmalloc( n + 1 );
 	}
 #ifdef USE_GNUPG_ICONV
         else if(use_iconv) {
@@ -992,7 +1000,7 @@ utf8_to_native( const char *string, size_t length, int delim )
             if (cd == (iconv_t)-1)
                 {
                     handle_iconv_error (active_charset_name, "utf-8", 1);
-                    m_free (buffer);
+                    xfree (buffer);
                     return utf8_to_native (string, length, delim);
                 }
 
@@ -1004,7 +1012,7 @@ utf8_to_native( const char *string, size_t length, int delim )
             outbytes = n * MB_LEN_MAX;
             if (outbytes / MB_LEN_MAX != n) 
                 BUG (); /* Actually an overflow. */
-            outbuf = outptr = m_alloc (outbytes);
+            outbuf = outptr = xmalloc (outbytes);
             if ( iconv (cd, (ICONV_CONST char **)&inptr, &inbytes,
                         &outptr, &outbytes) == (size_t)-1) {
                 static int shown;
@@ -1015,9 +1023,9 @@ utf8_to_native( const char *string, size_t length, int delim )
                 shown = 1;
                 /* Didn't worked out.  Temporary disable the use of
                  * iconv and fall back to our old code. */
-                m_free (buffer);
+                xfree (buffer);
                 buffer = NULL;
-                m_free (outbuf);
+                xfree (outbuf);
                 use_iconv = 0;
                 outbuf = utf8_to_native (string, length, delim);
                 use_iconv = 1;
@@ -1027,7 +1035,7 @@ utf8_to_native( const char *string, size_t length, int delim )
                 /* We could realloc the buffer now but I doubt that it makes
                    much sense given that it will get freed anyway soon
                    after.  */
-                m_free (buffer);
+                xfree (buffer);
             }
             iconv_close (cd);
             return outbuf;
@@ -1040,6 +1048,111 @@ utf8_to_native( const char *string, size_t length, int delim )
     }
 }
 
+/* This is similar to native_to_utf8, except it can take any input
+   (which may or may not be UTF8 encoded) and return something that is
+   (almost) definitely UTF8.  This code is mostly borrowed from
+   GPA. */
+
+char *
+string_to_utf8 (const char *string)
+{
+  const char *s;
+  
+  if (!string)
+    return NULL;
+  
+  /* Due to a bug in old and not so old PGP versions user IDs have
+     been copied verbatim into the key.  Thus many users with Umlauts
+     et al. in their name will see their names garbled.  Although this
+     is not an issue for me (;-)), I have a couple of friends with
+     Umlauts in their name, so let's try to make their life easier by
+     detecting invalid encodings and convert that to Latin-1.  We use
+     this even for X.509 because it may make things even better given
+     all the invalid encodings often found in X.509 certificates.  */
+  for (s = string; *s && !(*s & 0x80); s++)
+    ;
+  if (*s && ((s[1] & 0xc0) == 0x80) && ( ((*s & 0xe0) == 0xc0)
+                                         || ((*s & 0xf0) == 0xe0)
+                                         || ((*s & 0xf8) == 0xf0)
+                                         || ((*s & 0xfc) == 0xf8)
+                                         || ((*s & 0xfe) == 0xfc)) )
+    {  
+      /* Possible utf-8 character followed by continuation byte.
+         Although this might still be Latin-1 we better assume that it
+         is valid utf-8. */
+      return xstrdup (string);
+     }
+  else if (*s && !strchr (string, 0xc3))
+    {
+      size_t length=0;
+      char *buffer,*p;
+
+      /* No 0xC3 character in the string; assume that it is Latin-1.  */
+
+      for(s=string; *s; s++ ) 
+        {
+          length++;
+          if( *s & 0x80 )
+            length++;
+	}
+      buffer = xmalloc( length + 1 );
+      for(p=buffer, s=string; *s; s++ )
+        {
+          if( *s & 0x80 )
+            {
+              *p++ = 0xc0 | ((*s >> 6) & 3);
+              *p++ = 0x80 | ( *s & 0x3f );
+            }
+          else
+            *p++ = *s;
+        }
+      *p = 0;
+
+      return buffer;
+    }
+  else
+    {
+      /* Everything else is assumed to be UTF-8.  We do this even that
+         we know the encoding is not valid.  However as we only test
+         the first non-ascii character, valid encodings might
+         follow.  */
+      return xstrdup (string);
+    }
+}
+
+/* Same as asprintf but return an allocated buffer suitable to be
+   freed using xfree.  This function simply dies on memory failure,
+   thus no extra check is required. */
+char *
+xasprintf (const char *fmt, ...)
+{
+  va_list ap;
+  char *buf;
+
+  va_start (ap, fmt);
+  if (estream_vasprintf (&buf, fmt, ap) < 0)
+    log_fatal ("estream_asprintf failed: %s\n", strerror (errno));
+  va_end (ap);
+  return buf;
+}
+
+/* Same as above but return NULL on memory failure.  */
+char *
+xtryasprintf (const char *fmt, ...)
+{
+  int rc;
+  va_list ap;
+  char *buf;
+
+  va_start (ap, fmt);
+  rc = estream_vasprintf (&buf, fmt, ap);
+  va_end (ap);
+  if (rc < 0)
+    return NULL;
+  return buf;
+}
+
+
 /****************************************************
  ******** locale insensitive ctype functions ********
  ****************************************************/
@@ -1056,76 +1169,6 @@ ascii_islower (int c)
     return c >= 'a' && c <= 'z';
 }
 
-int 
-ascii_toupper (int c)
-{
-    if (c >= 'a' && c <= 'z')
-        c &= ~0x20;
-    return c;
-}
-
-int 
-ascii_tolower (int c)
-{
-    if (c >= 'A' && c <= 'Z')
-        c |= 0x20;
-    return c;
-}
-
-
-int
-ascii_strcasecmp (const char *a, const char *b)
-{
-  const unsigned char *p1 = (const unsigned char *)a;
-  const unsigned char *p2 = (const unsigned char *)b;
-  unsigned char c1, c2;
-
-  if (p1 == p2)
-    return 0;
-
-  do
-    {
-      c1 = ascii_tolower (*p1);
-      c2 = ascii_tolower (*p2);
-
-      if (c1 == '\0')
-	break;
-
-      ++p1;
-      ++p2;
-    }
-  while (c1 == c2);
-  
-  return c1 - c2;
-}
-
-int 
-ascii_strncasecmp (const char *a, const char *b, size_t n)
-{
-  const unsigned char *p1 = (const unsigned char *)a;
-  const unsigned char *p2 = (const unsigned char *)b;
-  unsigned char c1, c2;
-
-  if (p1 == p2 || !n )
-    return 0;
-
-  do
-    {
-      c1 = ascii_tolower (*p1);
-      c2 = ascii_tolower (*p2);
-
-      if ( !--n || c1 == '\0')
-	break;
-
-      ++p1;
-      ++p2;
-    }
-  while (c1 == c2);
-  
-  return c1 - c2;
-}
-
-
 int
 ascii_memcasecmp( const char *a, const char *b, size_t n )
 {
@@ -1156,56 +1199,6 @@ stpcpy(char *a,const char *b)
 }
 #endif
 
-
-#ifndef HAVE_STRSEP
-/* code taken from glibc-2.2.1/sysdeps/generic/strsep.c */
-char *
-strsep (char **stringp, const char *delim)
-{
-  char *begin, *end;
-
-  begin = *stringp;
-  if (begin == NULL)
-    return NULL;
-
-  /* A frequent case is when the delimiter string contains only one
-     character.  Here we don't need to call the expensive `strpbrk'
-     function and instead work using `strchr'.  */
-  if (delim[0] == '\0' || delim[1] == '\0')
-    {
-      char ch = delim[0];
-
-      if (ch == '\0')
-        end = NULL;
-      else
-        {
-          if (*begin == ch)
-            end = begin;
-          else if (*begin == '\0')
-            end = NULL;
-          else
-            end = strchr (begin + 1, ch);
-        }
-    }
-  else
-    /* Find the end of the token.  */
-    end = strpbrk (begin, delim);
-
-  if (end)
-    {
-      /* Terminate the token and set *STRINGP past NUL character.  */
-      *end++ = '\0';
-      *stringp = end;
-    }
-  else
-    /* No more delimiters; this is the last token.  */
-    *stringp = NULL;
-
-  return begin;
-}
-#endif /*HAVE_STRSEP*/
-
-
 #ifndef HAVE_STRLWR
 char *
 strlwr(char *s)
@@ -1247,111 +1240,6 @@ strncasecmp( const char *a, const char *b, size_t n )
 
 
 #ifdef _WIN32
-/* 
- * Like vsprintf but provides a pointer to malloc'd storage, which
- * must be freed by the caller (m_free).  Taken from libiberty as
- * found in gcc-2.95.2 and a little bit modernized.
- * FIXME: Write a new CRT for W32.
- */
-int
-vasprintf (char **result, const char *format, va_list args)
-{
-  const char *p = format;
-  /* Add one to make sure that it is never zero, which might cause malloc
-     to return NULL.  */
-  int total_width = strlen (format) + 1;
-  va_list ap;
-
-  /* this is not really portable but works under Windows */
-  memcpy ( &ap, &args, sizeof (va_list));
-
-  while (*p != '\0')
-    {
-      if (*p++ == '%')
-	{
-	  while (strchr ("-+ #0", *p))
-	    ++p;
-	  if (*p == '*')
-	    {
-	      ++p;
-	      total_width += abs (va_arg (ap, int));
-	    }
-	  else
-            {
-              char *endp;  
-              total_width += strtoul (p, &endp, 10);
-              p = endp;
-            }
-	  if (*p == '.')
-	    {
-	      ++p;
-	      if (*p == '*')
-		{
-		  ++p;
-		  total_width += abs (va_arg (ap, int));
-		}
-	      else
-                {
-                  char *endp;
-                  total_width += strtoul (p, &endp, 10);
-                  p = endp;
-                }
-	    }
-	  while (strchr ("hlL", *p))
-	    ++p;
-	  /* Should be big enough for any format specifier except %s
-             and floats.  */
-	  total_width += 30;
-	  switch (*p)
-	    {
-	    case 'd':
-	    case 'i':
-	    case 'o':
-	    case 'u':
-	    case 'x':
-	    case 'X':
-	    case 'c':
-	      (void) va_arg (ap, int);
-	      break;
-	    case 'f':
-	    case 'e':
-	    case 'E':
-	    case 'g':
-	    case 'G':
-	      (void) va_arg (ap, double);
-	      /* Since an ieee double can have an exponent of 307, we'll
-		 make the buffer wide enough to cover the gross case. */
-	      total_width += 307;
-	    
-	    case 's':
-	      total_width += strlen (va_arg (ap, char *));
-	      break;
-	    case 'p':
-	    case 'n':
-	      (void) va_arg (ap, char *);
-	      break;
-	    }
-	}
-    }
-  *result = m_alloc (total_width);
-  if (*result != NULL)
-    return vsprintf (*result, format, args);
-  else
-    return 0;
-}
-
-int
-asprintf (char **buf, const char *fmt, ...)
-{
-  int status;
-  va_list ap;
-
-  va_start (ap, fmt);
-  status = vasprintf (buf, fmt, ap);
-  va_end (ap);
-  return status;  
-}
-
 const char *
 w32_strerror (int w32_errno)
 {
